<?php

namespace BmbyForms\Interceptors;

use function BmbyForms\Requests\bf_create_lead_request;

/**
 * @return void
 */
function bf_cf7_form_submit_interceptor($contact_form, $abort, $submission) {
	global $bfAllWordpressAndBmbyFormElements;

	$formTitle = $contact_form->title();
	$formId = $contact_form->id;
	$formDescriptor = base64_encode("{$formId}:Contact Form 7:{$formTitle}");
	$formData = get_option("bf_match_{$formDescriptor}");
	if (!$formData) {
		//if no corresponding form data in a table intercept nothing and just return the form class
		return $contact_form;
	}

	if (empty($formData['form_title']) || empty($formData['fields'])) {
		//if data is corrupted no interception
		return $contact_form;
	}

	$formElements = [];
	foreach ($bfAllWordpressAndBmbyFormElements as $formElement) {
		$formElements[$formElement['id']] = $formElement['bmby_field'];
	}

	$postedData = $submission->get_posted_data();
	$dataToSend = [];
	foreach ($formData['fields'] as $key => $value) {
		if ($value < 0) {
			$dataToSend[$formElements[$value]] = $postedData[$key];
		}
		else {
			$dataToSend['DynVars'][$value] = $postedData[$key];
			$dataToSend['DynMongoVars'][$value] = $postedData[$key];
		}
	}

	$responseBody = bf_create_lead_request($dataToSend);

	if (!empty($responseBody['clientID']) && get_option('bf_product_api_key', null) && $metaField = get_option('bf_meta_field', null)) {
		$loggedInUser = wp_get_current_user();
		if (!is_super_admin($loggedInUser->ID) && !in_array('administrator', $loggedInUser->roles)) {
			$existedClientIDs = get_user_meta($loggedInUser->ID, $metaField);
			if (!$existedClientIDs || !in_array($responseBody['clientID'], $existedClientIDs)) {
				add_metadata('user', $loggedInUser->ID, $metaField, $responseBody['clientID']);
			}

			if ($responseBody['isReturningCustomer'] == true && $returningUserRole = get_option('bf_returning_user_role')) {
				$loggedInUser->add_role($returningUserRole);
			}
		}
	}
}
