<?php

namespace BmbyForms\Pages\FormFieldsPage;

use function BmbyForms\bf_get_redirect_url;

function bf_form_fields_page() {
	require_once __DIR__ . '/../../../bmby-check-api-data.php';
	global $bfAllWordpressAndBmbyFormElements;
	$redirectUrl = bf_get_redirect_url();

	if (isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'])) {
		if (!$formData = json_decode(gzuncompress(base64_decode($_GET['bf_forms'])), true)) {
			$_SESSION['type'] = translate('error', 'bmby-forms');
			$_SESSION['flashMessage'] = translate('Invalid input', 'bmby-forms');
			wp_redirect("{$redirectUrl}?page=bmby-forms%2Fwordpress-forms.php");
			exit();
		}

//		$form = get_post($formData['page_id']);
//		if (empty($formData['page_id']) || $form->post_status !== 'publish') {
//			$_SESSION['type'] = translate('error', 'bmby-forms');
//			$_SESSION['flashMessage'] = translate('Form is not published', 'bmby-forms');
//			//if php redirect is used I get warning that headers are already sent and not redirect happens
//			echo '<script type="text/javascript"> window.location.href = "../../../../../../wp-admin/admin.php?page=bmby-forms%2Fwordpress-forms.php"; </script>';
//			exit();
//		}

		if (!$formElements = !empty($formData['form_data']) ? $formData['form_data'] : []) {
			$_SESSION['type'] = translate('error', 'bmby-forms');
			$_SESSION['flashMessage'] = translate('This form does not contain input elements that can be correspondent to bmby elements', 'bmby-forms');
			wp_redirect("{$redirectUrl}?page=bmby-forms%2Fwordpress-forms.php");
			exit();
		}

		$bmbyFieldsList = array_map(function ($value) {
			return array('id' => $value['id'], 'bmby_field' => $value['bmby_field'], 'table' => isset($value['table']) ? $value['table'] : null);
		}, $bfAllWordpressAndBmbyFormElements);

		$dynVars = json_decode(get_option('bf_dyn_vars'), true);
		if ($dynVars) {
			//append dyn var options to usual ones
			$bmbyFieldsList = array_merge($bmbyFieldsList, array_map(function ($value) {
				if (isset($value['DynVarOptions']) && is_array($value['DynVarOptions']) && count($value['DynVarOptions']) > 0) {
					return array(
						'id' => $value['DynTitleID'],
						'bmby_field' => sprintf('%s (%s, %d options)', $value['Title'], strtolower($value['Type']), count($value['DynVarOptions'])),
						'selected' => false
					);
				}
				else {
					return array(
						'id' => $value['DynTitleID'],
						'bmby_field' => sprintf('%s (%s)', $value['Title'], strtolower($value['Type'])),
						'selected' => false
					);
				}
			}, $dynVars));

			//get options for Options column
			$options = [];
			foreach ($dynVars as $dynVar) {
				$optionsOfOneDynVar = [];
				if (isset($dynVar['DynVarOptions']) && is_array($dynVar['DynVarOptions']) && count($dynVar['DynVarOptions']) > 0) {
					foreach ($dynVar['DynVarOptions'] as $option) {
						$optionsOfOneDynVar[$option['DynOptionID']] = $option['Title'];
					}
				}

				$options[$dynVar['DynTitleID']] = $optionsOfOneDynVar;
			}
		}

		$savedFormData = get_option("bf_match_{$formData['form_descriptor']}");
		$savedFormFields = !empty($savedFormData) ? $savedFormData['fields'] : [];

		$comparisonCondition = isset($_GET['orderby']) && isset($_GET['dir']) ? "orderby={$_GET['orderby']}&dir={$_GET['dir']}" : null;
		switch ($comparisonCondition) {
			case 'orderby=type&dir=asc':
				uasort($formElements, function ($item1, $item2) {
					if ($item1['type'] == $item2['type']) {
						return 0;
					}

					return $item1['type'] <  $item2['type'] ? -1 : 1;
				});

				$_GET['orderby'] = 'type';
				$_GET['dir'] = 'desc';
				$typeHeaderClasses = 'sorted asc';
				$typeSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));

				$_GET['orderby'] = 'label';
				$_GET['dir'] = 'asc';
				$labelHeaderClasses = 'sortable asc';
				$labelSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));

				break;
			case 'orderby=type&dir=desc':
				uasort($formElements, function ($item1, $item2) {
					if ($item1['type'] == $item2['type']) {
						return 0;
					}

					return $item1['type'] > $item2['type'] ? -1 : 1;
				});

				$_GET['orderby'] = 'type';
				$_GET['dir'] = 'asc';
				$typeHeaderClasses = 'sorted desc';
				$typeSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));

				$_GET['orderby'] = 'label';
				$_GET['dir'] = 'asc';
				$labelHeaderClasses = 'sortable asc';
				$labelSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));


				break;
			case 'orderby=label&dir=asc':
				uasort($formElements, function ($item1, $item2) {
					if ($item1['label'] == $item2['label']) {
						return 0;
					}

					return $item1['label'] < $item2['label'] ? -1 : 1;
				});

				$_GET['orderby'] = 'type';
				$_GET['dir'] = 'asc';
				$typeHeaderClasses = 'sortable asc';
				$typeSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));

				$_GET['orderby'] = 'label';
				$_GET['dir'] = 'desc';
				$labelHeaderClasses = 'sorted asc';
				$labelSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));


				break;
			case 'orderby=label&dir=desc':
				uasort($formElements, function ($item1, $item2) {
					if ($item1['label'] == $item2['label']) {
						return 0;
					}

					return $item1['label'] > $item2['label'] ? -1 : 1;
				});

				$_GET['orderby'] = 'type';
				$_GET['dir'] = 'asc';
				$typeHeaderClasses = 'sortable asc';
				$typeSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));

				$_GET['orderby'] = 'label';
				$_GET['dir'] = 'asc';
				$labelSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));

				$labelHeaderClasses = 'sorted desc';

				break;

			default:
				$_GET['orderby'] = 'type';
				$_GET['dir'] = 'asc';
				$typeHeaderClasses = 'sortable desc';
				$typeSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));

				$_GET['orderby'] = 'label';
				$_GET['dir'] = 'asc';
				$labelHeaderClasses = 'sortable desc';
				$labelSortUrl = sprintf('%s://%s/wp-admin/admin.php?%s', isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http', $_SERVER['HTTP_HOST'], http_build_query($_GET));

		}
	} else {
		$_SESSION['type'] = translate('error', 'bmby-forms');
		$_SESSION['flashMessage'] = translate('Invalid input', 'bmby-forms');
		wp_redirect("{$redirectUrl}?page=bmby-forms%2Fwordpress-forms.php"); exit();
	}

	require 'form_fields_page.view.html';
}
