<?php

namespace BmbyForms\Pages\PaymentFormPage;

/**
 * @return void
 */
function bf_payment_form_page() {
	global $wpdb;
	global $mbeatServer;

	try {
		//get success page
		$successPageID = get_option('bf_success_page', null);
		if ($successPageID) {
			$successPageUrl = get_permalink($successPageID);
		}

		//get error page
		$errorPageID = get_option('bf_error_page', null);
		if ($errorPageID) {
			$errorPageUrl = get_permalink($errorPageID);
		}

		if (!$productApiKey = get_option('bf_product_api_key', null)) {
			echo "<script>window.location.href = '{$errorPageUrl}?error=no_product_api_key';</script>";
			exit();
		}

		$userId = get_current_user_id();
		if (!$userId) {
			echo "<script>window.location.href = '{$errorPageUrl}?error=no_logged_user';</script>";
			exit();
		}

		$products = [];
		$order = wc_create_order(['customer_id' => $userId]);
		$order->update_status("Pending payment", 'Order', true);
		$productInput = json_decode(gzuncompress(base64_decode($_REQUEST['products'])), true);
		foreach ($productInput as $product) {
			$args = [
				'id' => ['filter' => FILTER_VALIDATE_INT, 'options' => ['min_range' => 1]],
				'post_id' => ['filter' => FILTER_VALIDATE_INT, 'options' => ['min_range' => 1]],
				'quantity' => ['filter' => FILTER_VALIDATE_INT, 'options' => ['min_range' => 1]],
				'price' => ['filter' => FILTER_VALIDATE_FLOAT, 'options' => ['min_range' => 0]]
			];

			$product = filter_var_array($product, $args);
			if (!$product)
				continue;

			if (in_array(false, $product, true) || in_array(null, $product, true))
				continue;

			$productObject = wc_get_product($product['post_id']);
			$args = array(
				'name' => $productObject->get_name(),
				'product_id' => $product['post_id'],
				'subtotal' => $product['price'] * $product['quantity'],
				'total' => $product['price'] * $product['quantity'],
				'quantity' => $product['quantity'],
			);

			//$args must hold totals
			$order->add_product($productObject, $product['quantity'], $args);

			$products[] = $product;
		}

		$products = array_filter($products);
		if (!$products) {
			echo "<script>window.location.href = '{$errorPageUrl}?error=no_products';</script>";
			exit();
		}

		$order->calculate_totals();

		$metaFieldName = get_option('bf_meta_field', null);
		if (!$metaFieldName) {
			echo "<script>window.location.href = '{$errorPageUrl}?error=no_product_api_key';</script>";
			exit();
		}


		//get user client id, created during lead creating (right after user register)
		$sql ="SELECT meta_value FROM {$wpdb->usermeta} um WHERE um.user_id = %d AND um.meta_key = %s AND meta_value REGEXP '^[0-9]+$' ORDER BY umeta_id DESC LIMIT 1;";
		$data = $wpdb->get_results($wpdb->prepare($sql, $userId, $metaFieldName));
		if (!$clientID = $data[0]->meta_value) {
			echo "<script>window.location.href = '{$errorPageUrl}?error=no_usermeta_client_id';</script>";
			exit();
		}

		$response = wp_remote_post( "{$mbeatServer}/api/products/price-offers", [
			'data_format' => 'body',
			'headers' => [
				'Accept' => 'application/json',
				'Content-Type' => 'application/json',
				'Authorization' => "Bearer {$productApiKey}"
			],
			'body' => json_encode(array('clientId' => $clientID, 'products' => $products)),
			'sslverify' => false
		]);

		$result = json_decode($response['body'], true);
		if ($response['response']['code'] >= 400 || !empty($result['error'])) {
			echo "<script>window.location.href = '{$errorPageUrl}?error=price_offer_create_error';</script>";
			exit();
		}

		$priceOfferID = filter_var(trim($result['priceOfferID']), FILTER_VALIDATE_INT, ["options" => ["min_range" => 1]]);
		if (!$priceOfferID) {
			echo "<script>window.location.href = '{$errorPageUrl}?error=price_offer_invalid';</script>";
			exit();
		}

		$successPageUrlEncoded = urlencode(sprintf('%s?purchase_course_id=%s&order_id=%d', $successPageUrl, implode(',', array_column($products, 'id')), $order->get_id()));
		$errorPageUrlEncoded = urlencode($errorPageUrl);
		$requestUrl = sprintf("https://www.bmby.com/invoicing/index.php?view=public-payment&api-key=%s&proposal_id=%s&success_url=%s&error_url=%s",
			$productApiKey, $priceOfferID, $successPageUrlEncoded, $errorPageUrlEncoded
		);

		$response = wp_remote_get($requestUrl, array('sslverify' => false));
		if (is_a($response, 'WP_Error')) {
			echo "<script>window.location.href = '{$errorPageUrl}?error=payment_gateway_request_error';</script>";
			exit();
		}

		$zap = json_decode($response['body'], true);
		if ($zap['status'] == 'success' && isset($zap['formUrl'])) {
			echo "<script>window.location.href = '{$zap['formUrl']}';</script>";
		} elseif ($zap['status'] == 'error' && isset($zap['error'])) {
			echo "<script>window.location.href = '{$errorPageUrl}?error={$zap['error']}';</script>";
		} else {
			echo "<script>window.location.href = '{$errorPageUrl}?error=payment_gateway_invalid_response';</script>";
		}
	} catch (\Exception $e) {
		wp_die('Error while creating payment form');
	}
}
