<?php

namespace BmbyForms\Pages\ProductsPage;

use function BmbyForms\bf_get_redirect_url;
use function BmbyForms\Taxonomies\create_custom_terms;

/**
 * @return void
 */
function bf_import_products_post_request_handler() {
	require __DIR__ . '/../../../bmby-check-api-data.php';
	$redirectUrl = bf_get_redirect_url();

	if (isset($_POST['_wpnonce']) && wp_verify_nonce($_POST['_wpnonce'])) {
		global $mbeatServer;
		global $wpdb;

		$productApiKey = get_option('bf_product_api_key');

		//get api data
		try {
			$response = wp_remote_get("{$mbeatServer}/api/products/keys/product-validation?lead_api_key={$leadApiKey}", [
				'sslverify' => false,
				'headers'   => [
					'Accept' => 'application/json',
					'Content-Type' => 'application/json',
					'Authorization' => "Bearer {$productApiKey}"
				]
			]);

			$zap = json_decode($response['body'], true);
			if ($response['response']['code'] >= 400 || !empty($zap['error'])) {
				$_SESSION['type'] = translate('error', 'bmby-forms');
				$error = !empty($zap['error']) ? $zap['error'] : 'There is no bmby data associated with this api key';
				$_SESSION['flashMessage'] = translate($error, 'bmby-forms');
				header('Location: ' . esc_url("{$redirectUrl}?page=bmby-products%2Fproducts.php"));
				exit();
			}

			//otherwise, lead api key and product api key are from same project, go on
			$response = wp_remote_get("{$mbeatServer}/api/products", [
				'sslverify' => false,
				'headers'   => [
					'Accept' => 'application/json',
					'Content-Type' => 'application/json',
					'Authorization' => "Bearer {$productApiKey}"
				]
			]);

			$courses = json_decode($response['body'], true, 512, JSON_UNESCAPED_UNICODE);
			if ($response['response']['code'] >= 400 || !empty($courses['error'])) {
				$_SESSION['type'] = translate('error', 'bmby-forms');
				$error = !empty($courses['error']) ? $courses['error'] : 'There is no bmby data associated with this api key';
				$_SESSION['flashMessage'] = translate($error, 'bmby-forms');
				header('Location: ' . esc_url("{$redirectUrl}?page=bmby-products%2Fproducts.php"));
				exit();
			}

			$noErrors = true;
			$productPostType = get_option('bf_product_post_type') ?: 'bmby_course';
			$meetingPostType = get_option('bf_meeting_post_type') ?: 'bmby_meeting';

			$sites = [];
			$semesters = [];
			$lecturers = [];
			$timeSlots = [];
			$specialities = [];
			$courseAndMeetingIDs = [];
			foreach($courses as $course) {
				//get old post_content or post_content filtered to save them to new course
				$metaProductID = get_option('bf_meta_product_product_id') ?: 'productId';   //get meta_key name for productID
				$sql = "SELECT p.ID, p.post_content, p.post_content_filtered FROM {$wpdb->posts} p INNER JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id 
					WHERE pm.meta_key = '%s' AND pm.meta_value = '%s' ORDER BY p.ID DESC LIMIT 1;";
				$oldCourseDescriptionData = $wpdb->get_results($wpdb->prepare($sql, $metaProductID, $course['productId']));

				//remove previous course post and postmeta data if present
				if ($oldCourseDescriptionData) {
					$sql = $wpdb->prepare( "DELETE p, pm
													FROM {$wpdb->posts} p
													INNER JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id
													INNER JOIN (
													    SELECT post_id 
													    FROM {$wpdb->postmeta} 
													    WHERE meta_key = '%s' AND meta_value = '%s'
													    ORDER BY post_id DESC
													    LIMIT 1
													) subquery ON pm.post_id = subquery.post_id
													WHERE pm.post_id = subquery.post_id;", $metaProductID, $course['productId']);
					$wpdb->query($sql);
				}

				$courseID = wp_insert_post([
					'post_title' => preg_replace('/"|”|״/', '', str_replace(['"', '”', '״', '"'], '', $course['title'])),
					'post_status' => 'publish',
					'post_type' => $productPostType,
					'comment_status' => 'closed',
					'ping_status' => 'closed',
					'post_name' => sanitize_title($course['title']),
					'post_content' => isset($oldCourseDescriptionData[0]->post_content) ? $oldCourseDescriptionData[0]->post_content : '',
					'post_content_filtered' => isset($oldCourseDescriptionData[0]->post_content_filtered) ? $oldCourseDescriptionData[0]->post_content_filtered : ''
				]);

				if (!$courseID)
					$noErrors = false;

				$mediatorID = !empty($course['site']) ? $course['site'] : $course['siteId'];
				if (!empty($mediatorID) && !in_array($courseID, $sites[$mediatorID])) {
					$sites[$mediatorID][] = $courseID;
				}

				if (!empty($course['timeSlot']) && !in_array($courseID, $timeSlots[$course['timeSlot']])) {
					$timeSlots[$course['timeSlot']][] = $courseID;
				}

				$semester = !empty($course['semester']) ? $course['semester'] : $course['semesterId'];
				if (!empty($semester) && !in_array($courseID, $semesters[$semester])) {
					$semesters[$semester][] = $courseID;
				}

				//add post meta which is course data
				foreach($course as $key => $value) {
					if ($key === 'title') {
						continue;
					} else if ($key !== 'meetings' && $key !== 'specialities') {
						$optionName = sprintf('bf_meta_product_%s', strtolower(preg_replace("/([a-z])([A-Z])/", "$1_$2", $key)));
						$metaName = get_option($optionName) ?: $key;
						$metaID = add_post_meta($courseID, $metaName, $value, true);
						if (!$metaID)
							$noErrors = false;
					} else if ($key === 'specialities') {
						foreach ($course['specialities'] as $specialityID => $speciality) {
							$metaID = add_post_meta($courseID, 'specialityID', $specialityID, false);
							if (!$metaID)
								$noErrors = false;

							$metaID = add_post_meta($courseID, 'speciality', $speciality, false);
							if (!$metaID)
								$noErrors = false;

							$specialityMediator = !empty($speciality) ? $speciality : $specialityID;
							if (!in_array($courseID, $specialities[$specialityMediator])) {
								$specialities[$specialityMediator][] = $courseID;
							}
						}
					} else {
						foreach($course['meetings'] as $meeting) {
							//get old meeting data
							$sql = "SELECT p.ID, p.post_content, p.post_content_filtered FROM {$wpdb->posts} p INNER JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id 
										WHERE pm.meta_key = '%s' AND pm.meta_value = '%s' ORDER BY p.ID DESC LIMIT 1;";
							$oldMeetingDescriptionData = $wpdb->get_results($wpdb->prepare($sql, 'meetingId', $meeting['meetingId']));

							//remove previous meeting post and postmeta data if present
							if ($oldMeetingDescriptionData) {
								$sql = $wpdb->prepare( "DELETE p, pm
													FROM {$wpdb->posts} p
													INNER JOIN {$wpdb->postmeta} pm ON p.ID = pm.post_id
													INNER JOIN (
													    SELECT post_id 
													    FROM {$wpdb->postmeta} 
													    WHERE meta_key = '%s' AND meta_value = '%s'
													    ORDER BY post_id DESC
													    LIMIT 1
													) subquery ON pm.post_id = subquery.post_id
													WHERE pm.post_id = subquery.post_id;", 'meetingId', $meeting['meetingId']);
								$wpdb->query($sql);
							}

							$meetingData = [
								'post_title' => $meeting['name'],
								'post_status' => 'publish',
								'post_parent' => $courseID,
								'post_type' => $meetingPostType,
								'comment_status' => 'closed',
								'ping_status' => 'closed',
								'post_name' => sanitize_title($meeting['name']),
								'post_content' => isset($oldMeetingDescriptionData[0]->post_content) ? $oldMeetingDescriptionData[0]->post_content : '',
								'post_content_filtered' => isset($oldMeetingDescriptionData[0]->post_content_filtered) ? $oldMeetingDescriptionData[0]->post_content_filtered : ''
							];

							if (preg_match('/^\d+\s*מפגש|מפגש\s*\d+$/u', $meeting['name']) === 1 || strpos($meeting['name'], 'ללא') !== false) {
								continue;
							}

							foreach($meeting as $k => $v) {
								if ($k === 'name')
									continue;

								if ($k === 'meetingId')
									$meetingData['meta_input']['meetingId'] = $v;

								$optionName = sprintf('bf_meta_meeting_%s', strtolower(preg_replace("/([a-z])([A-Z])/", "$1_$2", $k)));
								$metaName = get_option($optionName) ?: $k;
								$meetingData['meta_input'][$metaName] = $v;
							}

							$meetingID = wp_insert_post($meetingData);
							if (!$meetingID)
								$noErrors = false;

							$mediatorID = !empty($meeting['lecturer']) ? str_replace(['”', '’', '"', "'"], '', $meeting['lecturer']) : $meeting['lecturerId'];
							if (!empty($mediatorID) && !in_array($courseID, $lecturers[$mediatorID])) {
								$lecturers[$mediatorID][] = $courseID;
							}

							$courseAndMeetingIDs[$courseID][] = $meetingID;
						}
					}
				}

				//also create _price meta field for woo commerce
				$metaID = add_post_meta($courseID, '_price', $course['price'], true);
				if (!$metaID)
					$noErrors = false;
			}

			if (is_plugin_active('jet-engine/jet-engine.php')
			    && class_exists('Jet_Engine_DB')
			    && $jetPostTypesTable = \Jet_Engine_DB::tables()['post_types']['name']
			) {
				$productPostType = get_option('bf_product_post_type') ?: 'bmby_course';
				$regexp = sprintf('"parent_object";s:%d:"%s"', strlen("posts::{$productPostType}"), "posts::{$productPostType}");
				$sql = <<<EOT
	select id from {$jetPostTypesTable} where status = %s AND args REGEXP '%s' LIMIT 1;
EOT;
				$data = $wpdb->get_results($wpdb->prepare($sql, 'relation', $regexp));
				if (!empty($data[0]->id)) {
					$relation = jet_engine()->relations->get_active_relations($data[0]->id); //$data[0]->id is a relation id
					$relation->set_update_context('child');
					foreach ($courseAndMeetingIDs as $courseID => $meetings) {
						foreach ($meetings as $meetingID) {
							$result = $relation->update($courseID, $meetingID);
							if (!$result) {
								$noErrors = true;
							}
						}
					}

				} else {
					$noErrors = true;
				}
			}

			create_custom_terms($sites, $lecturers, $specialities, $timeSlots, $semesters);

			if ($noErrors === false) {
				$_SESSION['type'] = translate('error', 'bmby-forms');
				$_SESSION['flashMessage'] = translate('Some of bmby products may not be saved due to errors', 'bmby-forms');
				header('Location: ' . esc_url("{$redirectUrl}?page=bmby-products%2Fproducts.php"));
				exit();
			}
		} catch (\Exception $e) {
			$_SESSION['type'] = translate('error', 'bmby-forms');
			$_SESSION['flashMessage'] = translate(sprintf('%s: %s', 'Error while product api data retrieval', $e->getMessage()), 'bmby-forms');
			header('Location: ' . esc_url("{$redirectUrl}?page=bmby-products%2Fproducts.php"));
			exit();
		}

		$_SESSION['flashMessage'] = translate('Bmby Products successfully imported', 'bmby-forms');
		header('Location: ' . esc_url("{$redirectUrl}?page=bmby-products%2Fproducts.php"));
	} else {
		$_SESSION['type'] = translate('error', 'bmby-forms');
		$_SESSION['flashMessage'] = translate('Invalid input data specified', 'bmby-forms');
		header('Location: ' . esc_url("{$redirectUrl}?page=bmby-products%2Fproducts.php"));
		exit();
	}
}
