<?php

namespace BmbyForms\Pages\ProductsPage;

use function BmbyForms\bf_get_redirect_url;

/**
 * @return void
 */
function bf_bmby_products_page() {
	global $wpdb;
	$redirectUrl = bf_get_redirect_url();

	try {
		if (!get_option('bf_product_api_key', null)) {
			wp_die(__('No valid product key found', 'bmby-forms'), __('Error', 'bmby-forms'), array(
				'response'  => 403,
				'back_link' => esc_url("{$redirectUrl}?page=bmby-forms%2Fsettings.php")
			));
		}

		$products = [];
		$productPostType = get_option('bf_product_post_type') ?: 'bmby_course';
		$metaProductID = get_option('bf_meta_product_product_id') ?: 'productId';
		$metaProductSku = get_option('bf_meta_product_sku') ?: 'sku';
		$metaProductPrice = get_option('bf_meta_product_price') ?: 'price';
		$metaProductStartDate = get_option('bf_meta_product_start_date') ?: 'startDate';
		$metaProductEndDate = get_option('bf_meta_product_end_date') ?: 'endDate';
		$sql = "SELECT 
					(SELECT meta_value FROM {$wpdb->postmeta} m WHERE m.post_id = p.ID AND meta_key = %s LIMIT 1) AS productId,
					(SELECT meta_value FROM {$wpdb->postmeta} m WHERE m.post_id = p.ID AND meta_key = %s LIMIT 1) AS sku, 
					p.post_title, 
					(SELECT meta_value FROM {$wpdb->postmeta} m WHERE m.post_id = p.ID AND meta_key = %s LIMIT 1) AS price, 
					(SELECT date_format(meta_value, %s)  FROM {$wpdb->postmeta} m WHERE m.post_id = p.ID AND meta_key = %s LIMIT 1) AS startDate, 
					(SELECT date_format(meta_value, %s) FROM {$wpdb->postmeta} m WHERE m.post_id = p.ID AND meta_key = %s LIMIT 1) AS endDate
					FROM {$wpdb->posts} p
					LEFT JOIN  {$wpdb->postmeta} pm ON p.ID = pm.post_id 
					WHERE p.post_status NOT IN ('trash', 'auto-draft') AND p.post_type = %s GROUP BY p.ID;";
		$products = $wpdb->get_results($wpdb->prepare($sql, $metaProductID, $metaProductSku,
			$metaProductPrice, '%d/%m/%Y', $metaProductStartDate, '%d/%m/%Y', $metaProductEndDate, $productPostType), ARRAY_A);

		//load customizable names for columns
		$columnId = get_option('bf_column_id') ?: 'ID';
		$columnCatalog = get_option('bf_column_catalog') ?: 'Catalog Number';
		$columnTitle = get_option('bf_column_title') ?: 'Title';
		$columnPrice = get_option('bf_column_price') ?: 'Price';
		$columnStartDate = get_option('bf_column_start_date') ?: 'Start Date';
		$columnEndDate = get_option('bf_column_end_date') ?: 'End Date';

		$comparisonCondition = isset($_GET['orderby']) && isset($_GET['dir']) ? "orderby={$_GET['orderby']}&dir={$_GET['dir']}" : null;
		switch ($comparisonCondition) {
			case 'orderby=id&dir=asc':
				uasort($products, function ($item1, $item2) {
					if ($item1['productId'] == $item2['productId']) {
						return 0;
					}

					return (int) $item1['productId'] < (int) $item2['productId'] ? -1 : 1;
				});

				$idSortDir = 'desc';
				$idHeaderClasses = 'sorted asc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';

				break;
			case 'orderby=id&dir=desc':
				uasort($products, function ($item1, $item2) {
					if ($item1['productId'] == $item2['productId']) {
						return 0;
					}

					return (int) $item1['productId'] > (int) $item2['productId'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sorted desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';

				break;
			case 'orderby=catalog-number&dir=asc':
				uasort($products, function ($item1, $item2) {
					if ($item1['sku'] == $item2['sku']) {
						return 0;
					}

					return $item1['sku'] < $item2['sku'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'desc';
				$catalogNameHeaderClasses = 'sorted asc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';

				break;
			case 'orderby=catalog-number&dir=desc':
				uasort($products, function ($item1, $item2) {
					if ($item1['sku'] == $item2['sku']) {
						return 0;
					}

					return $item1['sku'] > $item2['sku'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sorted desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';

				break;
			case 'orderby=title&dir=asc':
				uasort($products, function ($item1, $item2) {
					if ($item1['title'] == $item2['title']) {
						return 0;
					}

					return $item1['title'] < $item2['title'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'desc';
				$titleHeaderClasses = 'sorted asc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';

				break;
			case 'orderby=title&dir=desc':
				uasort($products, function ($item1, $item2) {
					if ($item1['title'] == $item2['title']) {
						return 0;
					}

					return $item1['title'] > $item2['title'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sorted desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';

				break;
			case 'orderby=price&dir=asc':
				uasort($products, function ($item1, $item2) {
					if ($item1['price'] == $item2['price']) {
						return 0;
					}

					return $item1['price'] < $item2['price'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'desc';
				$priceHeaderClasses = 'sorted asc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';

				break;
			case 'orderby=price&dir=desc':
				uasort($products, function ($item1, $item2) {
					if ($item1['price'] == $item2['price']) {
						return 0;
					}

					return $item1['price'] > $item2['price'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sorted desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';

				break;
			case 'orderby=start-date&dir=asc':
				uasort($products, function ($item1, $item2) {
					if ($item1['startDate'] == $item2['startDate']) {
						return 0;
					}

					return $item1['startDate'] < $item2['startDate'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'desc';
				$startDateHeaderClasses = 'sorted asc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';

				break;
			case 'orderby=start-date&dir=desc':
				uasort($products, function ($item1, $item2) {
					if ($item1['startDate'] == $item2['startDate']) {
						return 0;
					}

					return $item1['startDate'] > $item2['startDate'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sorted desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable asc';

				break;
			case 'orderby=end-date&dir=asc':
				uasort($products, function ($item1, $item2) {
					if ($item1['endDate'] == $item2['endDate']) {
						return 0;
					}

					return $item1['endDate'] < $item2['endDate'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'desc';
				$endDateHeaderClasses = 'sorted asc';

				break;
			case 'orderby=end-date&dir=desc':
				uasort($products, function ($item1, $item2) {
					if ($item1['endDate'] == $item2['endDate']) {
						return 0;
					}

					return $item1['endDate'] > $item2['endDate'] ? -1 : 1;
				});

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sorted desc';

				break;

			default:

				$idSortDir = 'asc';
				$idHeaderClasses = 'sortable desc';
				$catalogNameSortDir = 'asc';
				$catalogNameHeaderClasses = 'sortable desc';
				$titleSortDir = 'asc';
				$titleHeaderClasses = 'sortable desc';
				$priceSortDir = 'asc';
				$priceHeaderClasses = 'sortable desc';
				$startDateSortDir = 'asc';
				$startDateHeaderClasses = 'sortable desc';
				$endDateSortDir = 'asc';
				$endDateHeaderClasses = 'sortable desc';
		}
	} catch (\Exception $e) {
		$_SESSION['type'] = translate('error', 'bmby-forms');
		$_SESSION['flashMessage'] = translate(sprintf('%s: %s', 'Error while retrieving bmby products', $e->getMessage()), 'bmby-forms');
		header('Location: ' . esc_url("{$redirectUrl}?page=bmby-forms%2Fsettings.php")); exit();
	}

	require_once 'bmby_products_page.view.html';
}
