<?php

namespace BmbyForms\Pages\WordpressFormsPage;

use Exception;
use WPCF7_ContactForm;
use \ElementorPro\Plugin;
use RecursiveArrayIterator;
use RecursiveIteratorIterator;
use function BmbyForms\bf_is_plugin_active;
use function BmbyForms\bf_get_redirect_url;

/**
 * @throws Exception
 */
function bf_wordpress_forms_page() {
	require_once __DIR__ . '/../../../bmby-check-api-data.php';
	require_once  __DIR__ . '/../../../bmby-check-if-plugins-are-activated.php';
	global $cf7FormExcludedElements;
	global $elementorFormExcludedElements;
	$redirectUrl = bf_get_redirect_url();

	//if all form plugins are deactivated
	if (
		!bf_is_plugin_active('contact-form-7/wp-contact-form-7.php') &&
		( !bf_is_plugin_active('elementor/elementor.php') ||
		!bf_is_plugin_active('elementor-pro/elementor-pro.php') )
	) {
		wp_die(__('In order to use bmby forms plugin at least one of the plugins (elementor and elementor pro or contact form 7) must be activated', 'bmby-forms'),
			__('Error', 'bmby-forms'), array(
				'response' => 400,
				'back_link' => esc_url("{$redirectUrl}?plugin_status=all&paged=1&s")
			)
		);
	}

	$allForms = [];

	if (bf_is_plugin_active('contact-form-7/wp-contact-form-7.php')) {
		$args = array(
			'post_type'      => 'wpcf7_contact_form',
			'post_status'    => 'publish',
			'posts_per_page' => - 1,
			'order'          => 'ASC',
			'orderby'        => 'ID'
		);

		$cf7Forms = get_posts($args);

		foreach ($cf7Forms as $cf7Form) {
			$scannedTags = WPCF7_ContactForm::get_instance($cf7Form->ID)->scan_form_tags();
			$formElements = [];
			foreach ($scannedTags as $key => $scannedTag) {
				if (array_search($scannedTag->basetype, $cf7FormExcludedElements) !== false) {
					continue;
				}

				$label = !empty($scannedTag->labels) ? $scannedTag->labels[0] : ($scannedTag->name ?: $scannedTag->raw_name);
				$formElement = [
					'name' => base64_encode($scannedTag->raw_name),
					'label' => $label,
					'encoded_label' => base64_encode("{$label}:{$scannedTag->basetype}"),
					'type'  => $scannedTag->basetype
				];

				$formElements[] = $formElement;
			}

			$formTitle = WPCF7_ContactForm::get_instance($cf7Form->ID)->title();

			//check if match data is already present for this form
			$formDescriptor = base64_encode("{$cf7Form->ID}:Contact Form 7:{$formTitle}");
			$savedFormData = get_option("bf_match_{$formDescriptor}");
			$connectedFields = !empty($savedFormData['fields']) ? count($savedFormData['fields']) : 0;
			$disconnectedFields = count($formElements) - $connectedFields;

			//Form data is sent through url. If form is too large then 502 bad gateway error may occur.
			//Compression is needed to reduce size and evade this error. Increasing "buffer-size" nginx option might also help.
			$formData = base64_encode(gzcompress(json_encode([
				'page_id'      => $cf7Form->ID,
				'form_descriptor'  => $formDescriptor,
				'form_type'    => 'Contact Form 7',
				'form_title'   => $formTitle,
				'form_data'    => $formElements,
				'date_created' => (new \DateTime($cf7Form->post_date))->format('d/m/Y H:i:s')
			]), 9));

			$allForms[] = [
				'page_id'      => $cf7Form->ID,
				'form_type'    => 'Contact Form 7',
				'form_title'   => $formTitle,
				'form_data'    => $formData,
				'connected_fields'   => $connectedFields,
				'disconnected_fields'   => $disconnectedFields,
				'date_created' => $cf7Form->post_date,
				'date_created_formatted' => (new \DateTime($cf7Form->post_date))->format('d/m/Y H:i:s')
			];
		}
	}

	if (bf_is_plugin_active('elementor/elementor.php') && bf_is_plugin_active('elementor-pro/elementor-pro.php')) {
		$args = array(
//			'post_type'      => array('page', 'post'),
//			'post_type'      => 'elementor_library',
			'post_type'      => array('page', 'post', 'elementor_library'),
			'post_status'    => 'publish',
			'posts_per_page' => - 1,
			'order'          => 'ASC',
			'orderby'        => 'ID'
		);

		$allPublishedPages = get_posts($args);

		$elementorForms = array();
		foreach ($allPublishedPages as $page) {
			$document = Plugin::elementor()->documents->get($page->ID);

			if ($document) {
				$data = $document->get_elements_data();
				if ($data) {
					$arrayIterator = new RecursiveArrayIterator($data);
					$recursiveIterator = new RecursiveIteratorIterator($arrayIterator, RecursiveIteratorIterator::SELF_FIRST);

					foreach ($recursiveIterator as $key => $value) {
						if ($key === 'settings' && !empty($value['form_name']) && !empty($value['form_fields'])) {
							$formElements = [];
							foreach ($value['form_fields'] as $form_field) {
								if (isset($form_field['field_type']) && array_search($form_field['field_type'], $elementorFormExcludedElements) !== false) {
									continue;
								}

								$name = !empty($form_field['custom_id']) ? $form_field['custom_id'] : "field_{$form_field['_id']}";
								$label = !empty($form_field['field_label']) ? $form_field['field_label'] : (!empty($form_field['placeholder']) ? $form_field['placeholder'] : $name);
								$type = !empty($form_field['field_type']) ? $form_field['field_type'] : 'text';
								$formElement = [
									'name' => base64_encode($name),
									'label' => $label,
									'encoded_label' => base64_encode("{$label}:{$type}"),
									'type'  => $type,
								];

								$formElements[] = $formElement;
							}

							//check if match data is already present for this form
							$formDescriptor = base64_encode("{$page->ID}:Elementor Form:{$value['form_name']}");
							$savedFormData = get_option("bf_match_{$formDescriptor}");
							$connectedFields = !empty($savedFormData['fields']) ? count($savedFormData['fields']) : 0;
							$disconnectedFields = count($formElements) - $connectedFields;

							//Form data is sent through url. If form is too large then 502 bad gateway error may occur.
							//Compression is needed to reduce size and evade this error. Increasing "buffer-size" nginx option might also help.
							$formData = base64_encode(gzcompress(json_encode([
								'page_id'      => $page->ID,
								'form_descriptor'  => $formDescriptor,
								'form_type'    => 'Elementor Form',
								'form_title'   => $value['form_name'],
								'form_data'    => $formElements,
								'date_created' => (new \DateTime($page->post_date))->format('d/m/Y H:i:s'),
							]), 9));

							$allForms[] = [
								'page_id'      => $page->ID,
								'form_type'    => 'Elementor Form',
								'form_title'   => $value['form_name'],
								'form_data'    => $formData,
								'connected_fields'   => $connectedFields,
								'disconnected_fields'   => $disconnectedFields,
								'date_created' => $page->post_date,
								'date_created_formatted' => (new \DateTime($page->post_date))->format('d/m/Y H:i:s'),
							];
						}
					}
				}
			}
		}
	}

	$comparisonCondition = isset($_GET['orderby']) && isset($_GET['dir']) ? "orderby={$_GET['orderby']}&dir={$_GET['dir']}" : null;
	switch ($comparisonCondition) {
		case 'orderby=id&dir=asc':
			uasort($allForms, function ($item1, $item2) {
				if ($item1['page_id'] == $item2['page_id']) {
					return 0;
				}

				return (int) $item1['page_id'] < (int) $item2['page_id'] ? -1 : 1;
			});

			$idSortDir = 'desc';
			$idHeaderClasses = 'sorted asc';
			$formTypeSortDir = 'asc';
			$formTypeHeaderClasses = 'sortable asc';
			$formTitleSortDir = 'asc';
			$formTitleHeaderClasses = 'sortable asc';
			$dateCreatedSortDir = 'asc';
			$dateCreatedHeaderClasses = 'sortable asc';

			break;
		case 'orderby=id&dir=desc':
			uasort($allForms, function ($item1, $item2) {
				if ($item1['page_id'] == $item2['page_id']) {
					return 0;
				}

				return (int) $item1['page_id'] > (int) $item2['page_id'] ? -1 : 1;
			});

			$idSortDir = 'asc';
			$idHeaderClasses = 'sorted desc';
			$formTypeSortDir = 'asc';
			$formTypeHeaderClasses = 'sortable asc';
			$formTitleSortDir = 'asc';
			$formTitleHeaderClasses = 'sortable asc';
			$dateCreatedSortDir = 'asc';
			$dateCreatedHeaderClasses = 'sortable asc';

			break;
		case 'orderby=form-type&dir=asc':
			uasort($allForms, function ($item1, $item2) {
				if ($item1['form_type'] == $item2['form_type']) {
					return 0;
				}

				return $item1['form_type'] < $item2['form_type'] ? -1 : 1;
			});

			$idSortDir = 'asc';
			$idHeaderClasses = 'sortable asc';
			$formTypeSortDir = 'desc';
			$formTypeHeaderClasses = 'sorted asc';
			$formTitleSortDir = 'asc';
			$formTitleHeaderClasses = 'sortable desc';
			$dateCreatedSortDir = 'asc';
			$dateCreatedHeaderClasses = 'sortable desc';

			break;
		case 'orderby=form-type&dir=desc':
			uasort($allForms, function ($item1, $item2) {
				if ($item1['form_type'] == $item2['form_type']) {
					return 0;
				}

				return $item1['form_type'] > $item2['form_type'] ? -1 : 1;
			});

			$idSortDir = 'asc';
			$idHeaderClasses = 'sortable asc';
			$formTypeSortDir = 'asc';
			$formTypeHeaderClasses = 'sorted desc';
			$formTitleSortDir = 'asc';
			$formTitleHeaderClasses = 'sortable asc';
			$dateCreatedSortDir = 'asc';
			$dateCreatedHeaderClasses = 'sortable asc';

			break;
		case 'orderby=form-title&dir=asc':
			uasort($allForms, function ($item1, $item2) {
				if ($item1['form_title'] == $item2['form_title']) {
					return 0;
				}

				return $item1['form_title'] < $item2['form_title'] ? -1 : 1;
			});

			$idSortDir = 'asc';
			$idHeaderClasses = 'sortable asc';
			$formTypeSortDir = 'asc';
			$formTypeHeaderClasses = 'sortable desc';
			$formTitleSortDir = 'desc';
			$formTitleHeaderClasses = 'sorted asc';
			$dateCreatedSortDir = 'asc';
			$dateCreatedHeaderClasses = 'sortable asc';

			break;
		case 'orderby=form-title&dir=desc':
			uasort($allForms, function ($item1, $item2) {
				if ($item1['form_title'] == $item2['form_title']) {
					return 0;
				}

				return $item1['form_title'] > $item2['form_title'] ? -1 : 1;
			});

			$idSortDir = 'asc';
			$idHeaderClasses = 'sortable asc';
			$formTypeSortDir = 'asc';
			$formTypeHeaderClasses = 'sortable desc';
			$formTitleSortDir = 'asc';
			$formTitleHeaderClasses = 'sorted desc';
			$dateCreatedSortDir = 'asc';
			$dateCreatedHeaderClasses = 'sortable asc';

			break;
		case 'orderby=date-created&dir=asc':
			uasort($allForms, function ($item1, $item2) {
				if ($item1['date_created'] == $item2['date_created']) {
					return 0;
				}

				return $item1['date_created'] < $item2['date_created'] ? -1 : 1;
			});

			$idSortDir = 'asc';
			$idHeaderClasses = 'sortable asc';
			$formTypeSortDir = 'asc';
			$formTypeHeaderClasses = 'sortable desc';
			$formTitleSortDir = 'asc';
			$formTitleHeaderClasses = 'sortable desc';
			$dateCreatedSortDir = 'desc';
			$dateCreatedHeaderClasses = 'sorted asc';

			break;
		case 'orderby=date-created&dir=desc':
			uasort($allForms, function ($item1, $item2) {
				if ($item1['date_created'] == $item2['date_created']) {
					return 0;
				}

				return $item1['date_created'] > $item2['date_created'] ? -1 : 1;
			});

			$idSortDir = 'asc';
			$idHeaderClasses = 'sortable asc';
			$formTypeSortDir = 'asc';
			$formTypeHeaderClasses = 'sortable desc';
			$formTitleSortDir = 'asc';
			$formTitleHeaderClasses = 'sortable desc';
			$dateCreatedSortDir = 'asc';
			$dateCreatedHeaderClasses = 'sorted desc';

			break;

		default:
			$idSortDir = 'asc';
			$idHeaderClasses = 'sortable desc';
			$formTypeSortDir = 'asc';
			$formTypeHeaderClasses = 'sortable desc';
			$formTitleSortDir = 'asc';
			$formTitleHeaderClasses = 'sortable desc';
			$dateCreatedSortDir = 'asc';
			$dateCreatedHeaderClasses = 'sortable desc';
	}

	require 'wordpress_forms_page.view.html';
}
