<?php

namespace BmbyForms\Taxonomies;

/**
 * @param $sites
 * @param $lecturers
 * @param $specialities
 * @param $timeSlots
 * @param $semesters
 */
function create_custom_terms($sites, $lecturers, $specialities, $timeSlots, $semesters) {
	global $wpdb;

	foreach ($sites as $site => $courseIDs) {
		if ($term = get_term_by('slug', urlencode($site), 'sites')) {
			$termID = $term->term_id;
		} else {
			$mediator = wp_insert_term(
				$site,
				'sites',
				array(
					'slug' => urlencode($site)
				)
			);
			if (is_array($mediator) && isset($mediator['term_id'])) {
				$termID = $mediator['term_id'];
			}
		}

		foreach ($courseIDs as $courseID) {
			wp_set_object_terms($courseID, $termID, 'sites');
		}
	}

	foreach ($lecturers as $lecturer => $courseIDs) {
		if ($term = get_term_by('slug', urlencode($lecturer), 'lecturers')) {
			$termID = $term->term_id;
			$termTaxonomyID = $term->term_taxonomy_id;
		} else {
			$mediator = wp_insert_term(
				$lecturer,
				'lecturers',
				array(
					'slug' => urlencode($lecturer)
				)
			);

			if (is_array($mediator) && isset($mediator['term_taxonomy_id'])) {
				$termID = $mediator['term_id'];
				$termTaxonomyID = $mediator['term_taxonomy_id'];
			}
		}

		$count = count($courseIDs);

		$sql = $wpdb->prepare( "INSERT INTO {$wpdb->term_taxonomy} (term_id, description, taxonomy, count) VALUES (%s, %s, %s, %s)
			ON DUPLICATE KEY UPDATE count = %s;", $termID, '', 'lecturers', $count, $count);
		$wpdb->query($sql);

		$sql = "INSERT INTO {$wpdb->term_relationships} (object_id, term_taxonomy_id) VALUES ";
		foreach ($courseIDs as $k => $courseID) {
			if ($k < $count - 1) {
				$sql .= "({$courseID}, {$termTaxonomyID}), ";
			} else {
				$sql .= "({$courseID}, {$termTaxonomyID});";
			}
		}

		$wpdb->query($sql);
	}

	foreach ($specialities as $speciality => $courseIDs) {
		if ($term = get_term_by('slug', urlencode($speciality), 'specialities')) {
			$termID = $term->term_id;
		} else {
			$mediator = wp_insert_term(
				$speciality,
				'specialities',
				array(
					'slug' => urlencode($speciality)
				)
			);
			if (is_array($mediator) && isset($mediator['term_id'])) {
				$termID = $mediator['term_id'];
			}
		}

		foreach ($courseIDs as $courseID) {
			wp_set_object_terms($courseID, $termID, 'specialities', true);
		}
	}

	foreach ($timeSlots as $timeSlot => $courseIDs) {
		if ($term = get_term_by('slug', urlencode($timeSlot), 'time_slots')) {
			$termID = $term->term_id;
		} else {
			$mediator = wp_insert_term(
				$timeSlot,
				'time_slots',
				array(
					'slug' => urlencode($timeSlot)
				)
			);
			if (is_array($mediator) && isset($mediator['term_id'])) {
				$termID = $mediator['term_id'];
			}
		}

		foreach ($courseIDs as $courseID) {
			wp_set_object_terms($courseID, $termID, 'time_slots');
		}
	}

	foreach ($semesters as $semester => $courseIDs) {
		if ($term = get_term_by('slug', urlencode($semester), 'semesters')) {
			$termID = $term->term_id;
		} else {
			$mediator = wp_insert_term(
				$semester,
				'semesters',
				array(
					'slug' => urlencode($semester)
				)
			);
			if (is_array($mediator) && isset($mediator['term_id'])) {
				$termID = $mediator['term_id'];
			}
		}

		foreach ($courseIDs as $courseID) {
			wp_set_object_terms($courseID, $termID, 'semesters');
		}
	}

	//remove old relations data
//	$sql = $wpdb->prepare( "DELETE {$wpdb->term_relationships} FROM {$wpdb->term_taxonomy}
//                                    LEFT JOIN {$wpdb->terms} ON {$wpdb->terms}.term_id = {$wpdb->term_taxonomy}.term_id
//                                    LEFT JOIN {$wpdb->term_relationships} ON {$wpdb->term_relationships}.term_taxonomy_id = {$wpdb->term_taxonomy}.term_taxonomy_id
//                                    LEFT JOIN {$wpdb->termmeta} ON {$wpdb->termmeta}.term_id = {$wpdb->terms}.term_id
//                                    LEFT JOIN {$wpdb->posts} ON {$wpdb->posts}.ID = {$wpdb->term_relationships}.object_id
//									WHERE {$wpdb->term_taxonomy}.taxonomy IN (%s, %s, %s, %s) AND {$wpdb->posts}.ID IS NULL;", 'sites', 'lecturers', 'specialities', 'time_slots');
//	$wpdb->query($sql);
}
